% C.1.10 DOWNWARD CONTINUATION
%
% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO APPLY A DIRECTIONAL PATH FILTER
% INPUTS:
% a=TOTAL FIELD DATASET
% x=X-COORDINATES
% y=Y-COORDINATES
% req_x=NUMBER OF SAMPLES OF FFT IN THE X-DIRECTION
% req_y=NUMBER OF SAMPLES OF FFT IN THE Y-DIRECTION
% h=DISTANCE FOR CONTINUATION +VE=DOWNWARDS
% I=INCLINATION OF FIELD

% OUTPUTS:
% output_sym=CONTINUED DATASET

close all
clear all

% Script to perform upward and downward continuation 

a=load('/data/generate_synthetic/depths/inc90_ht0_2/0_1_by_0_1/depths'); % Total field gridded dataset

x=0:0.1:20; % x-coordinates
y=0:0.1:10; % y-coordinates

filename='noise_0_0';
min_spec=-50; % minimum power for spectrum plotting

h=0.5; % distance for continuation +ve=downwards
I=90; % inclination of field

plot_grid_imagesc_warn(x,y,a,'Input Data',999,999,'(nT)',[filename,'_input.png'])

% Number of samples for FFT (should be greater than 110% of a))
req_x=256;
req_y=256;

%%%% END OF USER INPUTS %%%%

size_a=size(a);
len_x=size_a(:,2);
len_y=size_a(:,1);

pad_x1=floor((req_x-len_x)/2);
pad_x2=ceil((req_x-len_x)/2);
pad_y1=floor((req_y-len_y)/2);
pad_y2=ceil((req_y-len_y)/2);

% Pad X

for count=1:len_y
    first_point=a(count,1);
    costap=tukeywin(pad_x1*2,1);
    mid_data(count,[1:pad_x1])=first_point*costap([1:pad_x1]); % Add padded part of X before data

    mid_data(count,[pad_x1+1:len_x+pad_x1])=a(count,:); % Add real data

    last_point=a(count,len_x);
    costap=tukeywin(pad_x2*2,1);
    mid_data(count,[len_x+pad_x1+1:len_x+pad_x1+pad_x2])=last_point*costap([pad_x2+1:pad_x2*2]); % Add padded part of X after data
end

size_mid=size(mid_data);
len_x=size_mid(:,2);
len_y=size_mid(:,1);

% Pad Y

end_count=size(mid_data);

for count=1:end_count(:,2)
    first_point=mid_data(1,count);
    costap=tukeywin(pad_y1*2,1);
    new_data([1:pad_y1],count)=first_point*costap([1:pad_y1]); % Add padded part of Y before data
    
    new_data([pad_y1+1:len_y+pad_y1],count)=mid_data(:,count); % Add real data
    
    last_point=mid_data(len_y,count);
    costap=tukeywin(pad_y2*2,1);
    new_data([len_y+pad_y1+1:len_y+pad_y1+pad_y2],count)=last_point*costap([pad_y2+1:pad_y2*2]); % Add padded part of X after data
end

% DC SHIFT

amount=mean(mean(new_data));
dc_shifted=new_data-amount;

size_prepared=size(dc_shifted);
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),dc_shifted,'Prepared Dataset',999,999,'nT',[filename,'_prepared.png'])

save([filename,'_prepared'],'dc_shifted','-ascii','-double')

% FFT

fft_result=fft2(dc_shifted);
fft_shift=fftshift(fft_result);
fft_display=(abs(fft_shift)/(size_prepared(:,2)*size_prepared(:,1))).^2;

x_max_wavenum=2*pi()/(2*(x(2)-x(1)));
y_max_wavenum=2*pi()/(2*(y(2)-y(1)));
x_max_sp_freq=1/(2*(x(2)-x(1)));
y_max_sp_freq=1/(2*(y(2)-y(1)));

wave_x=(x_max_wavenum*-1):(x_max_wavenum/(req_x/2)):x_max_wavenum-(x_max_wavenum/(req_x/2));
wave_y=(y_max_wavenum*-1):(y_max_wavenum/(req_y/2)):y_max_wavenum-(y_max_wavenum/(req_y/2));
sp_freq_x=(x_max_sp_freq*-1):(x_max_sp_freq/(req_x/2)):x_max_sp_freq-(x_max_sp_freq/(req_x/2));
sp_freq_y=(y_max_sp_freq*-1):(y_max_sp_freq/(req_y/2)):y_max_sp_freq-(y_max_sp_freq/(req_y/2));

plot_fft2_pcolor(wave_x,wave_y,log(abs(fft_display)),'FFT',min_spec,0,'Log FFT2',[filename,'_2d_spectrum.png'])

%%% DOWNWARD CONT %%%%

for countx=1:req_x
    for county=1:req_y
        squ2v2=sqrt((sp_freq_x(countx)^2)+(sp_freq_y(county)^2));
        expon=exp(2*pi()*h*squ2v2);
        filtered(county,countx)=fft_shift(county,countx)*expon;
    end
end

fft_display=(abs(filtered)/(size_prepared(:,2)*size_prepared(:,1))).^2;
plot_fft2_pcolor(wave_x,wave_y,log(abs(fft_display)),'FFT',min_spec,0,'Log FFT2',[filename,'_2d_spectrum.png'])

filtered((req_y/2)+1,(req_x/2)+1)=0;

unshift=ifftshift(filtered);
output=ifft2(unshift);
output_sym=ifft2(unshift,'symmetric');

plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),real(output),'Real',999,999,'Real',[filename,'_temp.png'])
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),imag(output),'Imaginary',999,999,'Imaginary',[filename,'_temp.png'])
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),output_sym,'Symmetrical',999,999,'Symmetrical',[filename,'_output_symmetric.png'])

output_sym(:,[1:pad_x1])=[];
output_sym([1:pad_y1],:)=[];
output_sym=output_sym(1:size_a(:,1),1:size_a(:,2));

plot_grid_imagesc_warn(x,y,output_sym,'Post Processed',-15,15,'nT',[filename,'_downcont_',num2str(h),'_post_processed.png'])

save([filename '_downcont_' num2str(h)],'output_sym','-ascii','-double')

