% C.1.11 FORWARD MODELLING
%
% 17/5/2012
% SAMUEL CHEYNEY - re-coded to matlab from Rao & Babu (1993)

% OUTPUTS:
% TCAL=FORWARD MODELLED DATASET

close all
clear all

% INPUTS:

NB=1; % Number of prisms
NY=101; % Number of observations in the x-direction
NX=101; % Number of observations in the y-direction
Y=0:0.1:10; % Distance to the anomaly point in the x-direction
X=0:0.1:10; % Distance to the anomaly point in the y-direction
AI=64*(pi/180); % Inclination of the geomagnetic field (radians)
AD=40*(pi/180); % Declination of the geomagnetic field (radians)
PAR1=[4,6,4,6,0.3,0.8,AI,AD,0,(0.0375e-3)]; % Interpreted values of each prism (A1, A2 (horizontal distances from the origin to the faces of a prism in the y-direction), B1, B2 (horizontal distances from the origin to the faces of a prism in the x-direction), H1 (depth to top of prism), H2 (depth to bottom of prism), AI0 (Inclination of the magnetisation vector of the prism (DEGREES!?), AD0 (Declination of the magnetisation vector of the prism (DEGREES!?), THI (Angle with respect to geographic north (degrees) and EI (Intensity of magnetisation (CGS UNITS))
CR1=0; % Constant regional field (gammas)
ITER=10; % Current iteration
ITER1=1; % Number of iterations used in the 1st stage
ITER2=5; % Number of iterations used in the 2nd stage
LT1=[1000 1000 1000]; % Limiting value of the domain in which the exact equation is used for anomaly calculation

% Interpreted below here

DY=Y(2)-Y(1); % Y-spacing
DX=X(2)-X(1); % X-spacing

I=1:NY; % List of number of observations in x
J=1:NX; % List of number of observations in y

TCAL=zeros(NY,NX); % Array of final results - set to zero

for IB=1:NB % IB=current prism
     A1 =PAR1(IB,1); % Horizontal distances from the origin to the faces of a prism in the y-direction
	 A2 =PAR1(IB,2); % Horizontal distances from the origin to the faces of a prism in the y-direction
	 B1 =PAR1(IB,3); % Horizontal distances from the origin to the faces of a prism in the x-direction
	 B2 =PAR1(IB,4); % Horizontal distances from the origin to the faces of a prism in the x-direction
	 H1 =PAR1(IB,5); % Depth to top of prism
	 H2 =PAR1(IB,6); % Depth to bottom of prism
	 AI0=PAR1(IB,7); % Inclination of the magnetisation vector of the prism (DEGREES!?)
	 AD0=PAR1(IB,8); % Declination of the magnetisation vector of the prism (DEGREES!?)
	 THI=PAR1(IB,9); % Angle with respect to geographic north (degrees)
	 EI =PAR1(IB,10); % Intensity of magnetisation (CGS UNITS)
	 CI =cos(AI); % Cosine of geomagnetic field inclination
	 SI =sin(AI); % Sine of geomagnetic field inclination
	 CI0=cos(AI0); % Cosine of geomagnetic prism inclination
	 SI0=sin(AI0); % Sine of geomagnetic prism inclination
	 CDT=cos(AD-THI); % Cosine of geomagnetic declination
	 SDT=sin(AD-THI); % Sine of geomagnetic declination
	 CD0T=cos(AD0-THI); % Cosine of prism declination
	 SD0T=sin(AD0-THI); % Sine of prism declination
	 SP=CI*CDT; % cos(inclination)*cos(declination) of geomagnetic field
	 SQ=CI*SDT; % cos(inclination)*sin(declination) of geomagnetic field
	 SR=SI; 
	 CL=CI0*CD0T; % cos(inclination)*cos(declination) of prism
	 CM=CI0*SD0T; % cos(inclination)*sin(declination) of prism
 	 CN=SI0;
	 G1=EI*(CM*SR+CN*SQ);
	 G2=EI*(CL*SR+CN*SP);
	 G3=EI*(CL*SQ+CM*SP);
	 G4=EI*(CN*SR-CM*SQ);
	 G5=EI*(CN*SR-CL*SP);
	 A =(A2-A1)*(B2-B1); % Cross sectional area of the prism
%	 MONITORING THE EYACT AND APPROYIMATE
%	 EQUATIONS FOR CALCULATION OF ANOMALIES
     if ITER <= ITER1 % If the current iteration number is less than ITER1
         LT=LT1(1); % Use the first value of LT1
     else
         if ITER <= ITER2 % If the current iteration number is less than ITER2
             LT=LT1(2); % Use the second value of LT1
         else
             LT=LT1(3); % If the current iteration number is greater than ITER1 & ITER2, use the third value of LT1
         end
     end
     
     RT1=LT*DY; % RT1=size of special area in y
	 RT2=LT*DX; % RT2=size of special area in x
     AM1=A1-RT1; % Extending the area outside the prism zone where the exact equation should be used
	 AM2=A2+RT1; % Extending the area outside the prism zone where the exact equation should be used
	 BM1=B1-RT2; % Extending the area outside the prism zone where the exact equation should be used
	 BM2=B2+RT2; % Extending the area outside the prism zone where the exact equation should be used
     
     Y1=NaN(NY);
     X1=NaN(NX);
     TCAL=zeros(NY,NX);
     TCAL1=zeros(NY,NX);

     for I=1:NY % Looping through Y
         for J=1:NX % Looping through X
             Y1(I)= Y(I)*cos(THI)+X(J)*sin(THI); % Will just equal the y-position if THI is zero
             X1(J)=-Y(I)*sin(THI)+X(J)*cos(THI); % Will just equal the x-position if THI is zero
             toto=AM1-Y1(I);
             tete=Y1(I)-AM2;
             tata=BM1-X1(J);
             tutu=X1(J)-BM2;
             if toto<0 % If Y is within the close area
                 if tete<0 % If Y is within the close area
                   if tata<0 % If X is within the close area
                       if tutu<0 % If X is within the close area
                           % CALCULATION OF ANOMALIES USING THE EYACT EQUATION
                           AL1=A1-Y1(I);
                           AL2=A2-Y1(I);
                           BT1=B1-X1(J);
                           BT2=B2-X1(J);
                           if AL1==0
                               AL1=0.000000000000000001;
                           end
                           if AL2==0
                               AL2=0.000000000000000001;
                           end
                           if BT1==0
                               BT1=0.000000000000000001;
                           end
                           if BT2==0
                               BT2=0.000000000000000001;
                           end
                           if H1==0
                               H1=0.000000000000000001;
                           end
                           if H2==0
                               H2=0.000000000000000001;
                           end
                           R1=sqrt(AL1*AL1+BT1*BT1+H1*H1);
                           R2=sqrt(AL1*AL1+BT1*BT1+H2*H2);
                           R3=sqrt(AL2*AL2+BT1*BT1+H1*H1);
                           R4=sqrt(AL2*AL2+BT1*BT1+H2*H2);
                           R5=sqrt(AL1*AL1+BT2*BT2+H1*H1);
                           R6=sqrt(AL1*AL1+BT2*BT2+H2*H2);
                           R7=sqrt(AL2*AL2+BT2*BT2+H1*H1);
                           R8=sqrt(AL2*AL2+BT2*BT2+H2*H2);
                           F11=R1+AL1;
                           F12=R2+AL1;
                           F13=R3+AL2;
                           F14=R4+AL2;
                           F15=R5+AL1;
                           F16=R6+AL1;
                           F17=R7+AL2;
                           F18=R8+AL2;
                            F1=log(F12*F13*F15*F18/(F11*F14*F16*F17));
                           F21=R1+BT1;
                           F22=R2+BT1;
                           F23=R3+BT1;
                           F24=R4+BT1;
                           F25=R5+BT2;
                           F26=R6+BT2;
                           F27=R7+BT2;
                           F28=R8+BT2;
                            F2=log(F22*F23*F25*F28/(F21*F24*F26*F27));
                           F31=R1+H1;
                           F32=R2+H2;
                           F33=R3+H1;
                           F34=R4+H2;
                           F35=R5+H1;
                           F36=R6+H2;
                           F37=R7+H1;
                           F38=R8+H2;
                            F3=log(F32*F33*F35*F38/(F31*F34*F36*F37));
                           F41=AL1*H1/(R1*BT1);
                           F42=AL1*H2/(R2*BT1);
                           F43=AL2*H1/(R3*BT1);
                           F44=AL2*H2/(R4*BT1);
                           F45=AL1*H1/(R5*BT2);
                           F46=AL1*H2/(R6*BT2);
                           F47=AL2*H1/(R7*BT2);
                           F48=AL2*H2/(R8*BT2);
                            F4=-atan(F41)+atan(F42)+atan(F43)-atan(F44)+atan(F45)-atan(F46)-atan(F47)+atan(F48);
                           F51=BT1*H1/(R1*AL1);
                           F52=BT1*H2/(R2*AL1);
                           F53=BT1*H1/(R3*AL2);
                           F54=BT1*H2/(R4*AL2);
                           F55=BT2*H1/(R5*AL1);
                           F56=BT2*H2/(R6*AL1);
                           F57=BT2*H1/(R7*AL2);
                           F58=BT2*H2/(R8*AL2);
                            F5=-atan(F51)+atan(F52)+atan(F53)-atan(F54)+atan(F55)-atan(F56)-atan(F57)+atan(F58);
                           TCAL1(I,J)=G1*F1+G2*F2+G3*F3+G4*F4+G5*F5;
                       else
                           % CALCULATION OF ANOMALIES USING THE APPROYIMATE EQUATION
                           AL=(A2+A1)/2.0-Y1(I);
                           BT=(B2+B1)/2.0-X1(J);
                           W =AL*AL+BT*BT;
                           R1=sqrt(W+H1*H1);
                           R2=sqrt(W+H2*H2);
                           R13=1.0/(R1*R1*R1);
                           R23=1.0/(R2*R2*R2);
                           C2=H1/R1-H2/R2;
                           C1=H2*R23-H1*R13-2.0*C2/W;
                           TCAL1(I,J)=(G1*BT+G2*AL)*(R13-R23)+G3*C1*AL*BT/W-G4*(C1*BT*BT+C2)/W-G5*(C1*AL*AL+C2)/W;
                           TCAL1(I,J)=A*TCAL1(I,J);
                       end
                   else
                       % CALCULATION OF ANOMALIES USING THE APPROYIMATE EQUATION
                       AL=(A2+A1)/2.0-Y1(I);
                       BT=(B2+B1)/2.0-X1(J);
                       W =AL*AL+BT*BT;
                       R1=sqrt(W+H1*H1);
                       R2=sqrt(W+H2*H2);
                       R13=1.0/(R1*R1*R1);
                       R23=1.0/(R2*R2*R2);
                       C2=H1/R1-H2/R2;
                       C1=H2*R23-H1*R13-2.0*C2/W;
                       TCAL1(I,J)=(G1*BT+G2*AL)*(R13-R23)+G3*C1*AL*BT/W-G4*(C1*BT*BT+C2)/W-G5*(C1*AL*AL+C2)/W;
                       TCAL1(I,J)=A*TCAL1(I,J);
                   end
                 else
                   % CALCULATION OF ANOMALIES USING THE APPROYIMATE EQUATION
                   AL=(A2+A1)/2.0-Y1(I);
                   BT=(B2+B1)/2.0-X1(J);
                   W =AL*AL+BT*BT;
                   R1=sqrt(W+H1*H1);
                   R2=sqrt(W+H2*H2);
                   R13=1.0/(R1*R1*R1);
                   R23=1.0/(R2*R2*R2);
                   C2=H1/R1-H2/R2;
                   C1=H2*R23-H1*R13-2.0*C2/W;
                   TCAL1(I,J)=(G1*BT+G2*AL)*(R13-R23)+G3*C1*AL*BT/W-G4*(C1*BT*BT+C2)/W-G5*(C1*AL*AL+C2)/W;
                   TCAL1(I,J)=A*TCAL1(I,J);
                 end
             else
               % CALCULATION OF ANOMALIES USING THE APPROYIMATE EQUATION
               AL=(A2+A1)/2.0-Y1(I);
               BT=(B2+B1)/2.0-X1(J);
               W =AL*AL+BT*BT;
               R1=sqrt(W+H1*H1);
               R2=sqrt(W+H2*H2);
               R13=1.0/(R1*R1*R1);
               R23=1.0/(R2*R2*R2);
               C2=H1/R1-H2/R2;
               C1=H2*R23-H1*R13-2.0*C2/W;
               TCAL1(I,J)=(G1*BT+G2*AL)*(R13-R23)+G3*C1*AL*BT/W-G4*(C1*BT*BT+C2)/W-G5*(C1*AL*AL+C2)/W;
               TCAL1(I,J)=A*TCAL1(I,J); 
             end
         end
     end
     
     for I=1:NY % Loop though Y
         for J=1:NX % Loop though X
             TCAL(I,J)=TCAL(I,J)+TCAL1(I,J); % Should just equal TCAL1 as TCAL should be zero before this
         end
     end
     
     % ADDITION OF CONSTANT REGIONAL FIELD ANOMALIES
     for I=1:NY
         for J=1:NX
             TCAL(I,J)=TCAL(I,J)+CR1; % Addition of constant value for regional field
         end
     end
end

% TCAL2=transpose(TCAL);
TCAL=TCAL*1e5;

plot_grid_pcolor(X,Y,TCAL,'Magnetic Anomaly',999,999,'nT','output.png')
             




             

