% C.1.13 iSPI
%
% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO CALCULATE iSPI
% INPUTS:
% a=TOTAL FIELD DATASET
% x=X-COORDINATES
% y=Y-COORDINATES
% i=ambient field inclination
% alpha=angle between the positive x-axis and magnetic north
% F=magnetisation of inducing field (nT)
% dz=VERTICAL GRADIENT
% theta_2nd_deriv=Local Phase (2nd Order)
% OUTPUTS:
% spi_si=SPI Structural Index
% abs=HORIZONTAL GRADIENT MAGNITUDE
% theta=Local Phase (1st Order)
% dtheta=Local Wavenumber (1st Order)
% mod_ind_dep=Model Independant Depth

close all
clear all

a=load('/data/generate_synthetic/cube/un_rotated/inc_90_height_0_2/rao_babu/101_samples/cube'); % Total field gridded dataset

x=0:0.1:10; % x-coordinates
y=0:0.1:10; % y-coordinates

i=90; % ambient field inclination
alpha=-90; % angle between the positive x-axis and magnetic north
F=47124; % magnetisation of inducing field (nT)

dz=load('../spi/fft_deriv/dz'); % USES FREQUENCY DOMAIN CALCULATED dZ

filename='cube_ispi_space';

plot_grid_imagesc_warn(x,y,a,'Input Data',999,999,'Magnetic Anomaly (nT)',[filename '_input.png'])

% Calculate horizontal gradient

[dx,dy,dh,xg,yg] = horizontal_gradient_magnitude_func(a,x,y);

% Calculate local phase

for county=1:length(yg)
    for countx=1:length(xg)
        theta(county,countx)=atan(dz(county+1,countx+1)/dh(county,countx));
    end
end

plot_grid_imagesc_warn_jet(xg,yg,theta,'Local Phase (1st Order)',999,999,'Local Phase (radians)',[filename '_local_phase_1st.png'])

% Calculate horizontal gradient of local phase = local wavenumber (See Smith 1998)

[temp1,temp2,dtheta,xgg,ygg] = horizontal_gradient_magnitude_func(theta,xg,yg);

plot_grid_imagesc_warn_jet(xgg,ygg,dtheta,'Local Wavenumber (1st Order)',999,999,'Local Wavenumber',[filename '_local_wavenumber_1st.png'])

% Calculate second derivative local wavenumber

dzdz=load('fft_deriv_dz/dz'); % USES FREQUENCY DOMAIN CALCULATED dZ

[dzdx,dzdy,dzdh,xg,yg] = man_grad_func(dz,x,y);

% Calculate local phase (2nd order)

for county=1:length(yg)
    for countx=1:length(xg)
        theta_2nd_deriv(county,countx)=atan(dzdz(county+1,countx+1)/dzdh(county,countx));
    end
end

plot_grid_imagesc_warn_jet(xg,yg,theta_2nd_deriv,'Local Phase (2nd Order)',999,999,'Local Phase (radians)',[filename '_local_phase_2nd.png'])

% Calculate local wavenumber (2nd order)

[temp1,temp2,dtheta_2nd_deriv,xgg,ygg] = man_grad_func(theta_2nd_deriv,xg,yg);

plot_grid_imagesc_warn_jet(xgg,ygg,dtheta_2nd_deriv,'Local Wavenumber (2nd Order)',999,999,'Local Wavenumber',[filename '_local_wavenumber_2nd.png'])

% Model independant depth

for county=1:length(ygg)
    for countx=1:length(xgg)
        mod_ind_dep(county,countx)=1/(dtheta_2nd_deriv(county,countx)-dtheta(county,countx));
    end
end

plot_grid_imagesc_warn_jet(xgg,ygg,mod_ind_dep,'Model Independant Depth',0,1,'Model Independant Depth (m)',[filename '_mod_ind_depth.png'])

cutoff=2.5;

mod_ind_dep_cutout=NaN(size(mod_ind_dep));

for county=1:length(ygg)
    for countx=1:length(xgg)
        if dtheta(county,countx)>cutoff
            mod_ind_dep_cutout(county,countx)=mod_ind_dep(county,countx);
        end
    end
end

plot_grid_pcolor_jet(xgg,ygg,mod_ind_dep_cutout,'Model Independant Depth',0,1,'Model Independant Depth (m)',[filename '_mod_ind_depth_cutout.png'])

% SPI Structural Index

for county=1:length(ygg)
    for countx=1:length(xgg)
        spi_si(county,countx)=(dtheta(county,countx)/(dtheta_2nd_deriv(county,countx)-dtheta(county,countx)))-1;
    end
end

plot_grid_imagesc_warn_jet(xgg,ygg,spi_si,'SPI Structural Index',0,2,'SPI Structural Index',[filename '_spi_si.png'])

spi_si_cutout=NaN(size(spi_si));

for county=1:length(ygg)
    for countx=1:length(xgg)
        if dtheta(county,countx)>cutoff
            spi_si_cutout(county,countx)=spi_si(county,countx);
        end
    end
end

plot_grid_pcolor_jet(xgg,ygg,spi_si_cutout,'SPI Structural Index',0,2,'SPI Structural Index',[filename '_spi_si_cutout.png'])


