% C.1.16 REDUCTION TO THE POLE

% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO CALCULATE REDUCED TO THE POLE DATASET
% INPUTS:
% a=TOTAL FIELD DATASET
% x=X-COORDINATES
% y=Y-COORDINATES
% req_x=NUMBER OF SAMPLES OF FFT IN THE X-DIRECTION
% req_y=NUMBER OF SAMPLES OF FFT IN THE Y-DIRECTION
% inc=INCLINATION OF FIELD 
% dec=DECLINATION OF FIELD

% OUTPUTS:
% rtp_sym=REDUCED TO THE POLE DATASET

close all
clear all

% Script for Reduction to the Pole

a=load('/data/generate_synthetic/cube/un_rotated/inc_60_height_0_2/cube');

x=0:0.1:10;
y=0:0.1:10;

filename='noise_0_0';
min_spec=-50;

plot_grid_imagesc_warn(x,y,a,'Input Data',999,999,'(nT)',[filename,'_input.png'])

% Number of Samples for FFT - Should be greater than 110% the number of samples of a. 

req_x=256;
req_y=256;

% Incomming inclination and declination

inc=60;
dec=0;

% End of user variables

size_a=size(a);
len_x=size_a(:,2);
len_y=size_a(:,1);

pad_x1=floor((req_x-len_x)/2);
pad_x2=ceil((req_x-len_x)/2);
pad_y1=floor((req_y-len_y)/2);
pad_y2=ceil((req_y-len_y)/2);

% Pad X

mid_data=NaN(length(y),req_x);

for count=1:len_y
    first_point=a(count,1);
    costap=tukeywin(pad_x1*2,1);
    mid_data(count,1:pad_x1)=first_point*costap(1:pad_x1); % Add padded part of X before data

    mid_data(count,pad_x1+1:len_x+pad_x1)=a(count,:); % Add real data

    last_point=a(count,len_x);
    costap=tukeywin(pad_x2*2,1);
    mid_data(count,len_x+pad_x1+1:len_x+pad_x1+pad_x2)=last_point*costap(pad_x2+1:pad_x2*2); % Add padded part of X after data
end

size_mid=size(mid_data);
len_x=size_mid(:,2);
len_y=size_mid(:,1);

% Pad Y

end_count=size(mid_data);

new_data=NaN(req_y,req_x);

for count=1:end_count(:,2)
    first_point=mid_data(1,count);
    costap=tukeywin(pad_y1*2,1);
    new_data(1:pad_y1,count)=first_point*costap(1:pad_y1); % Add padded part of Y before data
    
    new_data([pad_y1+1:len_y+pad_y1],count)=mid_data(:,count); % Add real data
    
    last_point=mid_data(len_y,count);
    costap=tukeywin(pad_y2*2,1);
    new_data([len_y+pad_y1+1:len_y+pad_y1+pad_y2],count)=last_point*costap([pad_y2+1:pad_y2*2]); % Add padded part of X after data
end

% DC SHIFT

amount=mean(mean(new_data));
dc_shifted=new_data-amount;

size_prepared=size(dc_shifted);

% save([filename,'_prepared'],'dc_shifted','-ascii','-double')

% FFT

fft_result=fft2(dc_shifted);
fft_shift=fftshift(fft_result);
fft_display=(abs(fft_shift)/(size_prepared(:,2)*size_prepared(:,1))).^2;

% fft_display=(abs(fft_shift)).^2;

x_max_wavenum=2*pi()/(2*(x(2)-x(1)));
y_max_wavenum=2*pi()/(2*(y(2)-y(1)));

wave_x=(x_max_wavenum*-1):(x_max_wavenum/(req_x/2)):x_max_wavenum-(x_max_wavenum/(req_x/2));
wave_y=(y_max_wavenum*-1):(y_max_wavenum/(req_y/2)):y_max_wavenum-(y_max_wavenum/(req_y/2));

plot_fft2_pcolor(wave_x,wave_y,log(abs(fft_display)),'FFT',min_spec,0,'Log FFT2',[filename,'_2d_spectrum.png'])

% % % % % REDUCTION TO THE POLE % % % % % 

h=sind(90-inc);
mz=cosd(90-inc);
my=h*cosd(dec);
mx=h*sind(dec);

a1=mz*mz-mx*mx;
a2=mz*mz-my*my;
a3=-my*mx-mx*my;
b1=mx*mz+mz*mx;
b2=my*mz+mz*my;

rtp=NaN(req_y,req_x);

for county=1:length(wave_y)
    for countx=1:length(wave_x)
        abs_ksq=(wave_y(county)^2)+(wave_x(countx)^2);
        a1kx2=a1*wave_x(countx)^2;
        a2ky2=a2*wave_y(county)^2;
        a3kxky=a3*wave_x(countx)*wave_y(county);
        abs_k=sqrt((wave_y(county)^2)+(wave_x(countx)^2));
        b1kx=b1*wave_x(countx);
        b2ky=b2*wave_y(county);
        b1kx_plus_b2ky=b1kx+b2ky;
        rtp(county,countx)=abs_ksq/(a1kx2+a2ky2+a3kxky+1i*abs_k*b1kx_plus_b2ky);
    end
end

fft_display=(abs(rtp)/(size_prepared(:,2)*size_prepared(:,1))).^2;
plot_fft2_pcolor(wave_x,wave_y,real(rtp),'FFT',999,999,'REAL',[filename,'_2d_spectrum_real.png'])
plot_fft2_pcolor(wave_x,wave_y,imag(rtp),'FFT',999,999,'IMAG',[filename,'_2d_spectrum_imag.png'])

% rtp(1,:)=0;
% rtp(:,1)=0;
% rtp(129,129)=0;

rtp(ceil(req_y/2)+1,ceil(req_x/2)+1)=0;

filtered=fft_shift.*rtp;

fft_display=(abs(filtered)/(size_prepared(:,2)*size_prepared(:,1))).^2;
plot_fft2_pcolor(wave_x,wave_y,log(abs(fft_display)),'FFT',min_spec,0,'Log FFT2',[filename,'_2d_spectrum_filtered.png'])

% MAKE SYMMETRIC
% 
% filtered(2:req_y/2+2,req_x/2+2:req_x)=9e99;
% filtered(req_y/2+2:req_y,req_x/2+1:req_x)=9e99;
% 
% mirrored_temp=flipud(filtered);
% mirrored=fliplr(mirrored_temp);
% 
% for countx=2:req_x
%     for county=2:req_y
%         if abs(filtered(county,countx))>9e98
%             testgrid(county,countx)=1;
%             filtered(county,countx)=complex(real(mirrored(county-1,countx-1)),imag(mirrored(county-1,countx-1)*-1));
%         end
%     end
% end

unshift=ifftshift(filtered);
rtp_result=ifft2(unshift);
rtp_sym=ifft2(unshift,'symmetric');

plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),real(rtp_result),'Real',999,999,'(nT/m)',[filename,'_real.png'])
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),imag(rtp_result),'Imag',999,999,'(nT/m)',[filename,'_imag.png'])
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),rtp_sym,'Symmetric',999,999,'(nT/m)',[filename,'_symmetric.png'])

rtp_sym(:,[1:pad_x1])=[];
rtp_sym([1:pad_y1],:)=[];
rtp_sym=rtp_sym(1:size_a(:,1),1:size_a(:,2));

plot_grid_imagesc_warn(x,y,rtp_sym,'Post Processed',999,999,'(nT/m)',[filename,'_post_processed.png'])

save('rtp','rtp_sym','-ascii','-double')
