% C.1.17 SPI
%
% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO CALCULATE SPI
% INPUTS:
% a=TOTAL FIELD DATASET
% x=X-COORDINATES
% y=Y-COORDINATES
% i=ambient field inclination
% alpha=angle between the positive x-axis and magnetic north
% F=magnetisation of inducing field (nT)
% dz=VERTICAL GRADIENT
% theta_2nd_deriv=Local Phase (2nd Order)
% OUTPUTS:
% theta=Local Phase (1st Order)
% dtheta=Local Wavenumber (1st Order)
% ldepth=Local Depth
% ldip=Local Dip
% ls=Local Susceptibility

close all
clear all

a=load('/data/generate_synthetic/cube/un_rotated/inc_90_height_0_2/rao_babu/101_samples/cube'); % Total field gridded dataset

x=0:0.1:10; % x-coordinates
y=0:0.1:10; % y-coordinates

i=90; % ambient field inclination
alpha=-90; % angle between the positive x-axis and magnetic north
F=47124; % magnitisation of inducing field (nT)

dz=load('fft_deriv/dz'); % USES FREQUENCY DOMAIN CALCULATED dZ

filename='cube_spi_space';

plot_grid_imagesc_warn(x,y,a,'Input Data',999,999,'Magnetic Anomaly (nT)',[filename '_input.png'])

% Calculate horizontal gradient

[dx,dy,dh,xg,yg] = horizontal_gradient_magnitude_func(a,x,y);

% Calculate local phase

for county=1:length(yg)
    for countx=1:length(xg)
        theta(county,countx)=atan(dz(county+1,countx+1)/dh(county,countx));
    end
end

plot_grid_imagesc_warn_jet(xg,yg,theta,'Local Phase',999,999,'Local Phase (radians)',[filename '_local_phase.png'])

% Calculate horizontal gradient of local phase = local wavenumber (See Smith 1998)

[temp1,temp2,dtheta,xgg,ygg] = horizontal_gradient_magnitude_func(theta,xg,yg);

plot_grid_imagesc_warn_jet(xgg,ygg,dtheta,'Local Wavenumber',999,999,'Local Wavenumber',[filename '_local_wavenumber.png'])

% Local Depth

ldepth=1./dtheta;

plot_grid_imagesc_warn_jet(xgg,ygg,ldepth,'Local Depth',0,2,'Local Depth (m)',[filename '_local_depth.png'])

% Local Dip

theta_deg=theta.*(180/pi());

I=atand(tand(i)/cosd(alpha));

for county=1:length(yg)
    for countx=1:length(xg)
        ldip(county,countx)=theta_deg(county,countx)+2*I-90;
    end
end
 
plot_grid_imagesc_warn_jet(xg,yg,ldip,'Local Dip',999,2,'Local Dip (degrees)','local_dip.png')

% Local Susceptibility

c=1-(cosd(i)^2)*(sind(alpha)^2);
dtheta_deg=dtheta.*(180/pi());

for county=1:length(yg)
    for countx=1:length(xg)
        analytic(county,countx)=sqrt(dx(county,countx)^2+dy(county,countx)^2+dz(county+1,countx+1)^2);
    end
end
        
for county=1:length(ygg)
    for countx=1:length(xgg)
        ls(county,countx)=analytic(county+1,countx+1)/(2*dtheta_deg(county,countx)*F*c*sind(ldip(county+1,countx+1)));
    end
end

plot_grid_imagesc_warn_jet(xgg,ygg,ls,'Local Susceptibility',0,1e-6,'Local Susceptibility','local_susceptibility.png')


