% C.1.19 TILT ANGLE
%
% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO CALCULATE TILT ANGLE
% INPUTS:
% orig_data=TOTAL FIELD DATASET
% upwrd_data=UPWARD CONTINUED OR UPPER SENSOR DATA
% analy=ANALYTIC SIGNAL AMPLITUDE
% x=X-COORDINATES
% y=Y-COORDINATES
% OUTPUTS:
% tiltd=TILT ANGLE (DEGREES)
% tilt=TILT ANGLE (RANDIANS)

close all
clear all

x=0:0.1:10; % Distance to the anomaly point in the x-direction
y=0:0.1:10; % Distance to the anomaly point in the y-direction

orig_data=load('/data/generate_synthetic/walls/negative/inc90_ht0_2/0_1_by_0_1/walls_neg'); % Total field gridded dataset
upwrd_dataload=importdata('/data/analytic_signal/manual_w_magpick/walls_neg/0_1_by_0_1/noise_none/upwrd_0_001.grd',' ',5); % Upward continued dataset (In this case in Surfer format/MAGPICK)
upwrd_data=upwrd_dataload.data;

[hx,hy,abs,xg,yg]=horizontal_gradient_magnitude_func(orig_data,x,y);

dz=(orig_data-upwrd_data)./0.001; % Distance of upward continued field

plot_grid_imagesc_warn(x,y,orig_data,'Raw Data - Height 0.2m',999,999,'Anomaly (nT)','data.png')

%%%%% ABSOLUTE HORIZONTAL GRADIENT %%%%%

plot_grid_imagesc_warn(xg,yg,abs,'Absolute Horizontal Gradient',999,999,'nT/m','horiz_grad.png')

%%%%% VERTICAL GRADIENT %%%%%%

plot_grid_imagesc_warn(x,y,dz,'Vertical Gradient',999,999,'nT/m','vert_grad.png')

%%%%% TILT ANGLE %%%%%

for county=2:length(y)-1
    for countx=2:length(y)-1
        tilt_ratio(county-1,countx-1)=dz(county,countx)/abs(county-1,countx-1);
        tilt(county-1,countx-1)=atan(tilt_ratio(county-1,countx-1));
    end
end

plot_grid_imagesc_warn(xg,yg,tilt,'Tilt Angle',999,999,'Radians','tilt_radians.png')

tiltd=tilt.*(180/pi());
plot_grid_imagesc_warn(xg,yg,tiltd,'Tilt Angle',999,999,'Degrees','tilt_degrees.png')

%%%%% OUTLINE PLOT %%%%%

figure(99)
contourf(xg,yg,tiltd,[-45 45],'w-');
caxis([-90 90])
load('MyColormaps','mycmap')
set(figure(99),'Colormap',mycmap)
title('Tilt Angle')
xlabel('Easting (m)')
ylabel('Northing (m)')
colorbar
ylabel(colorbar,'Tilt Angle (degrees)')

hold on
[C,h]=contour(xg,yg,tiltd,[0 0],'w--');
clabel(C,h,[0],'Color','w')

axis equal
axis([min(x) max(x) min(y) max(y)])

print('-dpng','-zbuffer','-r300','tilt_contour.png')

save('tilt_angle_degrees','tiltd','-ascii','-double')


