% C.1.1 ANALYTIC SIGNAL AMPLITUDE
%
% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO CALCULATE THE ANALYTIC SIGNAL AMPLITUDE FROM TOTAL FIELD DATA
% INPUTS:
% data=TOTAL FIELD DATASET
% upwrd=UPWARD CONTINUED OR UPPER SENSOR DATASET. IF NOT ALREADY CALCULATED USE SCRIPT C.1.10.
% x=X-COORDINATES
% y=Y-COORDINATES
% OUTPUTS:
% xg=X-COORDINATES OF ANALYTIC SIGNAL AMPLITUDE
% yg=Y-COORDINATES OF ANALYTIC SIGNAL AMPLITUDE
% analy=ANALYTIC SIGNAL AMPLITUDE

close all
clear all

%Script to manually calculate the analytic signal

data=load('../walls_neg'); % Total Field Dataset
upwrdload=importdata('../upwrd_0_001.grd',' ',5); % Pseudogravity dataset (magpick/surfer format)
upwrd=upwrdload.data;

x=0:0.1:10; % x-coordinates
y=0:0.1:10; % y-coordinates

[hx,hy,abs,xg,yg] = horizontal_gradient_magnitude_func(data,x,y); % Horizontal gradient magnitude function

dz=(data-upwrd)./0.001; % Distance of upward continuation

for countx=2:length(x)-1
    for county=2:length(y)-1
        analy(county-1,countx-1)=sqrt((hx(county-1,countx-1)^2)+(hy(county-1,countx-1)^2)+(dz(county,countx)^2));
    end
end
plot_grid_imagesc_warn(x,y,dz,'Manually calculated Vertical Derivative',999,999,'nT/m','man_dz.png')
plot_grid_imagesc_warn(xg,yg,analy,'Manually calculated Analytic Signal',999,999,'nT/m','man_analytic.png')

save('analy','analy','-ascii','-double')


