
% C.1.5 IDENTIFYING REMANENT MAGNETISATION USING PSEUDOGRAVITY
%
% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO CALCULATE THE CROSS-CORRELATION COEFFICIENT BETWEEN THE VERTICAL GRADIENT AND THE ANALYTIC SIGNAL AMPLITUDE FROM PSEUDOGRAVITY
% INPUTS:
% a=TOTAL FIELD DATASET
% x=X-COORDINATES
% y=Y-COORDINATES
% min_inc=MINIMUM INCLINATION TO BE INCLUDED
% max_inc=MAXIMUM INCLIMATION TO BE INCLUDED
% min_dec=MINIMUM DECLINATION TO BE INCLUDED
% max_dec=MAXIMUM DECLINATION TO BE INCLUDED
% req_x=NUMBER OF SAMPLES TO BE USED IN THE FFT
% req_y=NUMBER OF SAMPLES TO BE USED IN THE FFT
% OUTPUTS:
% output=XYZ FILE OF INCS,DECS,CROSS-CORRELATIONS

close all
clear all

a=load('/data/generate_synthetic/walls/rotated_30_degrees/inc_64_height_0_2/walls'); % Total field gridded dataset

x=0:0.1:10; % x-coordinates
y=0:0.1:10; % y-coordinates

count_list=1;

min_inc=20; % minimum inclination to be included
max_inc=90; % maximum inclination to be included
incr_inc=10; % inclination increment

min_dec=-180; % minimum declination to be included
max_dec=180; % maximum declination to be included
incr_dec=10; % declination increment

% Number of samples to be used in FFT
req_x=128; 
req_y=128;

A=1; % Constant for calculation of pseudogravity

%%%% END OF USER VARIABLES %%%%

for inc=min_inc:incr_inc:max_inc
    for dec=min_dec:incr_dec:max_dec
        
        [a_pgrav] = pgrav_function(a,x,y,req_x,req_y,inc,dec,A);
        
        [dx,dy,dz] = fft_deriv_function(a_pgrav,x,y,req_x,req_y);
        
        analy_sig=sqrt(dx.^2+dy.^2+dz.^2);
        
        dz_total=0;
        analy_total=0;
        count=0;

        for county=1:length(y)
            for countx=1:length(x)
                dz_total=dz_total+dz(county,countx);
                analy_total=analy_total+analy_sig(county,countx);
                count=count+1;
            end
        end

        dz_mean=dz_total/count;
        analy_mean=analy_total/count;

        sum_topline=0;
        sum_bot_v=0;
        sum_bot_t=0;
       
        for county=1:length(y)
            for countx=1:length(x)
                vj_min_v=dz(county,countx)-dz_mean;
                tj_min_t=analy_sig(county,countx)-dz_mean;
                top_line=vj_min_v*tj_min_t;
                sum_topline=sum_topline+top_line;
                
                sum_bot_v=sum_bot_v+((vj_min_v)^2);
                sum_bot_t=sum_bot_t+((tj_min_t)^2);
            end
        end
        
        cross_corr=sum_topline/sqrt(sum_bot_v*sum_bot_t);
        
        inc_list(count_list)=inc;
        dec_list(count_list)=dec;
        cross_corr_list(count_list)=cross_corr;
        count_list=count_list+1;

    end
end

% incs=10:40:90;
% decs=-180:40:180;
% 
% ZI = griddata(inc_list,dec_list,cross_corr_list,incs,decs');
% contourf(incs,decs,ZI)

incs=min_inc:incr_inc:max_inc;
decs=min_dec:incr_dec:max_dec;

output(:,1)=dec_list;
output(:,2)=inc_list;
output(:,3)=cross_corr_list;

save('dannemiller.output','output','-ascii','-double')

ZI = griddata(dec_list,inc_list,cross_corr_list,decs,incs');

figure
contourf(decs,incs,ZI)
% shading flat
xlabel('Declination (degrees)')
ylabel('Inclination (degrees)')
title('Dannemiller Method')
colorbar
ylabel(colorbar,['Cross Correlation'])
print('-dpng','-zbuffer','-r300','dannemiller.png')


