% C.1.8 DIRECTIONAL PASS/REJECT FILTER
%
% 17/5/2012
% SAMUEL CHEYNEY
%
% SCRIPT TO APPLY A DIRECTIONAL COSINE FILTER
% INPUTS:
% a=TOTAL FIELD DATASET
% x=X-COORDINATES
% y=Y-COORDINATES
% req_x=NUMBER OF SAMPLES OF FFT IN THE X-DIRECTION
% req_y=NUMBER OF SAMPLES OF FFT IN THE Y-DIRECTION
% taper_min_angle=MINIMUM ANGLE TO BEGIN COSINE TAPER
% cut_min_angle=MINIMUM ANGLE TO BEGIN CUT
% cut_max_angle=MAXIMUM ANGLE OF CUT
% taper_max_angle=MAXIMUM ANGLE OF COSINE TAPER
% cross_180=0; % =1 if filter zones crosses 180 degrees, otherwise =0.
% OUTPUTS:
% post_process=FILTERED DATASET

close all
clear all

% Script for direction pass/reject filtering

a=load('/data/generate_synthetic/compound/inc64_ht0_2/walls_tracks/walls_tracks_noise_none'); % Total field gridded dataset

x=0:0.1:10; % x-coordinates
y=0:0.1:10; % y-coordinates

filename='walls_tracks';
plot_grid_imagesc_warn(x,y,a,'Input Data',999,999,'nT',[filename '_input.png'])

% Number of samples for FFT (Should be greater than 110% of a))
req_x=128;
req_y=128;

taper_min_angle=150; % Minimum angle to begin cosine taper
cut_min_angle=160; % Minimum angle to begin cut
cut_max_angle=180; % Maximum angle of cut
taper_max_angle=190; % Maximum angle of cosine taper

cross_180=1; % =1 if filter zones crosses 180 degrees, otherwise =0.

%%%% END OF USER VARIABLES %%%%

size_a=size(a);
len_x=size_a(:,2);
len_y=size_a(:,1);

pad_x1=floor((req_x-len_x)/2);
pad_x2=ceil((req_x-len_x)/2);
pad_y1=floor((req_y-len_y)/2);
pad_y2=ceil((req_y-len_y)/2);

% Pad X

for count=1:len_y
    first_point=a(count,1);
    costap=tukeywin(pad_x1*2,1);
    mid_data(count,[1:pad_x1])=first_point*costap([1:pad_x1]); % Add padded part of X before data

    mid_data(count,[pad_x1+1:len_x+pad_x1])=a(count,:); % Add real data

    last_point=a(count,len_x);
    costap=tukeywin(pad_x2*2,1);
    mid_data(count,[len_x+pad_x1+1:len_x+pad_x1+pad_x2])=last_point*costap([pad_x2+1:pad_x2*2]); % Add padded part of X after data
end

size_mid=size(mid_data);
len_x=size_mid(:,2);
len_y=size_mid(:,1);

% Pad Y

end_count=size(mid_data);

for count=1:end_count(:,2)
    first_point=mid_data(1,count);
    costap=tukeywin(pad_y1*2,1);
    new_data([1:pad_y1],count)=first_point*costap([1:pad_y1]); % Add padded part of Y before data
    
    new_data([pad_y1+1:len_y+pad_y1],count)=mid_data(:,count); % Add real data
    
    last_point=mid_data(len_y,count);
    costap=tukeywin(pad_y2*2,1);
    new_data([len_y+pad_y1+1:len_y+pad_y1+pad_y2],count)=last_point*costap([pad_y2+1:pad_y2*2]); % Add padded part of X after data
end

% DC SHIFT

amount=mean(mean(new_data));
dc_shifted=new_data-amount;

size_prepared=size(dc_shifted);
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),dc_shifted,'Prepared Dataset',999,999,'nT',[filename,'_prepared.png'])

save([filename,'_prepared'],'dc_shifted','-ascii','-double')

% FFT

fft_result=fft2(dc_shifted);
fft_shift=fftshift(fft_result);
fft_display=(abs(fft_shift)/(size_prepared(:,2)*size_prepared(:,1))).^2;

% fft_display=(abs(fft_shift)).^2;

x_max_wavenum=2*pi()/(2*(x(2)-x(1)));
y_max_wavenum=2*pi()/(2*(y(2)-y(1)));

wave_x=(x_max_wavenum*-1):(x_max_wavenum/(req_x/2)):x_max_wavenum-(x_max_wavenum/(req_x/2));
wave_y=(y_max_wavenum*-1):(y_max_wavenum/(req_y/2)):y_max_wavenum-(y_max_wavenum/(req_y/2));

plot_fft2_pcolor(wave_x,wave_y,log(fft_display),'FFT',999,999,'Log FFT2',[filename,'_2d_spectrum.png'])
plot_fft2_pcolor(wave_x,wave_y,log(fft_display),'FFT',-30,0,'Log FFT2',[filename,'_2d_spectrum_scaled.png'])
save([filename '_fftdisplay'],'fft_display','-ascii','-double')
fft_orig=fft_display;

for countx=1:length(wave_x)
    for county=1:length(wave_y)
                
        if (wave_x(countx)>=0) && (wave_y(county)<0)
            if cross_180==1
                theta(county,countx)=atand(abs(wave_y(county))/abs(wave_x(countx)))+180;
            else
                theta(county,countx)=atand(abs(wave_y(county))/abs(wave_x(countx)));
            end
        end
        
        if (wave_x(countx)<0) && (wave_y(county)<0)
            theta(county,countx)=atand(abs(wave_x(countx))/abs(wave_y(county)))+90;
        end
       
        if (wave_x(countx)<0) && (wave_y(county)>=0)
            if cross_180==1
                theta(county,countx)=atand(abs(wave_y(county))/abs(wave_x(countx)))+180;
            else
                theta(county,countx)=atand(abs(wave_y(county))/abs(wave_x(countx)));
            end
        end
           
        if (wave_x(countx)>=0) && (wave_y(county)>=0)
            theta(county,countx)=atand(abs(wave_x(countx))/abs(wave_y(county)))+90;
        end
       
    end
end

plot_fft2_pcolor(wave_x,wave_y,theta,'Angle',999,999,'Angle',[filename,'_temp_angle.png'])

% FILTER

fft_filtered=fft_shift;
size_output=0;

for countx=1:length(wave_x)
    for county=1:length(wave_y)
        
        if (theta(county,countx)>=taper_min_angle) && (theta(county,countx)<cut_min_angle)
            taper_zone=cut_min_angle-taper_min_angle;
            diff=theta(county,countx)-taper_min_angle;
            percent=diff/taper_zone;
            factor=cos(percent*(0.5*pi()));
            fft_filtered(county,countx)=fft_filtered(county,countx)*factor;
        end
        
        if (theta(county,countx)>=cut_min_angle) && (theta(county,countx)<cut_max_angle)
            temp_imag=imag(fft_filtered(county,countx));
            fft_filtered(county,countx)=0;
        end
        
        if (theta(county,countx)>=cut_max_angle) && (theta(county,countx)<taper_max_angle)
            taper_zone=taper_max_angle-cut_max_angle;
            diff=theta(county,countx)-cut_max_angle;
            percent=1-(diff/taper_zone);
            factor=cos(percent*(0.5*pi()));
          
            fft_filtered(county,countx)=fft_filtered(county,countx)*factor;
            
        end
                
    end
end

fft_display=(abs(fft_filtered)/(size_prepared(:,2)*size_prepared(:,1))).^2;
plot_fft2_pcolor(wave_x,wave_y,log(fft_display),'FFT',-30,0,'Log FFT2',[filename,'_2d_spectrum_cut.png'])
save([filename '_filtered_fftdisplay'],'fft_display','-ascii','-double')

fft_new=fft_display;

filterplot=fft_new./fft_orig;
save([filename '_filter_factor'],'filterplot','-ascii','-double')
plot_grid_imagesc_warn(wave_x,wave_y,filterplot,'Filter',999,999,'Filter',[filename '_filter_factor.png'])

unshift=ifftshift(fft_filtered);
fft_display=(abs(unshift)/(size_prepared(:,2)*size_prepared(:,1))).^2;
plot_fft2_pcolor(wave_x,wave_y,log(fft_display),'FFT',-30,0,'Log FFT2',[filename '_unshift.png'])

unfft_symmetric=ifft2(unshift,'symmetric');
unfft_complex=ifft2(unshift);

plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),unfft_symmetric,'Inverse FFT - Symmetric',999,999,'nT',[filename '_inverse_symmetric.png'])
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),real(unfft_complex),'Inverse FFT - Real',999,999,'nT',[filename '_inverse_real.png'])
plot_grid_pcolor_prepared(1:size_prepared(:,2),1:size_prepared(:,1),imag(unfft_complex),'Inverse FFT - Imag',999,999,'nT',[filename '_inverse_imag.png'])

unfft_symmetric(:,[1:pad_x1])=[];
unfft_symmetric([1:pad_y1],:)=[];
post_process=unfft_symmetric(1:size_a(:,1),1:size_a(:,2));

plot_grid_imagesc_warn(x,y,post_process,'Post Processed Filtered',999,999,'nT',[filename '_filtered.png'])

save([filename '_filtered'],'post_process','-ascii','-double')

